@@ -104,20 +104,25 @@ logLik.unmarkedFit <- function(object, ...) {
 formula.unmarkedFit <- function (x, ...) x@formula
 
 getAllTerms.unmarkedFit <- function (x, intercept = FALSE, ...)  {
-  f <- formula(x)
-  t1 <- getAllTerms(f[[2]])
-  int1 <- attr(t1, "intercept")
-  if(intercept && int1) t1 <- c("Int", t1)
-  t2 <- getAllTerms(f[-2])
-  int2 <- attr(t2, "intercept")
-  if(intercept && int2) t2 <- c("Int", t2)
-  
-  structure(c(sprintf("psi(%s)",t1), sprintf("p(%s)",t2)), 
-            intercept=c(int1, int2),
-			interceptLabel=c("psi(Int)", "p(Int)")[as.logical(c(int1, int2))]
-			)
+	f <- formula(x)
+	res <- list()
+	while(is.call(f) && f[[1]] == "~") {
+		res <- c(f[c(1,length(f))], res)
+		f <- f[[2]]
+	}
+	names(res) <- sapply(x@estimates@estimates, slot, "short.name")
+	#res <- lapply(res, function(z) getAllTerms(call("~", z), intercept=FALSE))
+	res <- lapply(res, getAllTerms, intercept=FALSE)
+	attrInt <- sapply(res, attr, "intercept")
+	res <- unlist(lapply(names(res), function(i) sprintf("%s(%s)", i, res[[i]])))
+	Ints <- paste(names(attrInt[attrInt != 0]), "(Int)", sep="")
+	if(intercept) res <- c(Ints, res)
+	attr(res, "intercept") <- attrInt
+	attr(res, "interceptLabel") <-  Ints
+	res
 }
 
+
 #srcc <- function() sys.source("clipboard", .GlobalEnv)
 
 tTable.unmarkedFit <- function (model, ...) {
@@ -126,7 +131,7 @@ tTable.unmarkedFit <- function (model, ...) {
     rn <- rownames(ret)
     rn[rn=="(Intercept)"] <- "Int"
     rownames(ret) <- paste(y@short.name, "(", rn, ")", sep="")
-    ret 
+    ret
   }))
 }
 
